/** @file functions.i @brief luabindings for core generator functions */


/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2008  Thomas Moor
   Exclusive copyright is granted to Klaus Schmidt

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


/*
**************************************************
**************************************************
**************************************************

Functions misc

**************************************************
**************************************************
**************************************************
*/


// **** Version (does not participate in rti)
// Have function interface for a generator member
%{
void LuaVersion(const vGenerator& rGen, const std::string& ver, vGenerator& rRes) 
 { rGen.Version(ver,rRes);}
void LuaVersion(const vGenerator& rGen, const std::string& pat, const std::string& rep, vGenerator& rRes) 
 { rGen.Version(pat,rep,rRes);}
%}
// Tell SWIG with nice target name
%rename(Version) LuaVersion;
void LuaVersion(const vGenerator& rGen, const std::string& ver, vGenerator& rRes);
void LuaVersion(const vGenerator& rGen, const std::string& pat, const std::string& rep, vGenerator& rRes);

// *** Projection (non-determinsistic variant)
// Tell SWIG with nice target names
%rename(ProjectNonDet) ProjectNonDet;
void ProjectNonDet(vGenerator& rGen, const EventSet& rAlph);


/*
**************************************************
**************************************************
**************************************************

Functions from regex and friends

**************************************************
**************************************************
**************************************************
*/

// Tell SWIG (non-det versions dont participate with rti)
void LanguageConcatenateNonDet(const vGenerator& rGen1, const vGenerator& rGen2, vGenerator& rRes);
void LanguageUnionNonDet(const vGenerator& rGen1, const vGenerator& rGen2, vGenerator& rResGen);
void KleeneClosureNonDet(vGenerator& rGen);



/*
**************************************************
**************************************************
**************************************************

Functions related to EventSet

**************************************************
**************************************************
**************************************************
*/

// Have function interface with cEventSets
%{
void LuaEventSetUnion(const EventSet& rAlph1, const EventSet& rAlph2, EventSet& rRes) 
 { rRes = rAlph1; rRes.InsertSet(rAlph2); }
void LuaEventSetIntersection(const EventSet& rAlph1, const EventSet& rAlph2, EventSet& rRes)
 { rRes = rAlph1; rRes.RestrictSet(rAlph2); }
void LuaEventSetDifference(const EventSet& rAlph1, const EventSet& rAlph2, EventSet& rRes) 
 { rRes = rAlph1; rRes.EraseSet(rAlph2); }
%}



// Tell SWIG with nice target names (depreciated, use rti versions instead)
%rename(EventSetUnion) LuaEventSetUnion;
%rename(EventSetIntersection) LuaEventSetIntersection;
%rename(EventSetDifference) LuaEventSetDifference;
void LuaEventSetUnion(const EventSet& rAlph1, const EventSet& rAlph2, EventSet& rRes);
void LuaEventSetIntersection(const EventSet& rAlph1, const EventSet& rAlph2, EventSet& rRes);
void LuaEventSetDifference(const EventSet& rAlph1, const EventSet& rAlph2, EventSet& rRes);



/*
**************************************************
**************************************************
**************************************************

Help topics

**************************************************
**************************************************
**************************************************
*/



SwigHelpTopic("Functions","Core library functions");

SwigHelpEntry("Functions","regular expressions", " LanguageConcatenateNonDet(+In+ Generator Lm1, +In+ Generator Lm2, +Out+ Generator LRes)");
SwigHelpEntry("Functions","regular expressions", " LanguageUnion(+In+ Generator Lm1, +In+ Generator Lm2, +Out+ Generator LRes)");
SwigHelpEntry("Functions","regular expressions", " LanguageUnionNonDet(+In+ Generator Lm1, +In+ Generator Lm2, +Out+ Generator LRes)");
SwigHelpEntry("Functions","regular expressions", " LanguageIntersection(+In+ Generator Lm1, +In+ Generator Lm2, +Out+ Generator LRes)");
SwigHelpEntry("Functions","regular expressions", " KleeneClosureNonDet(+InOut+ Generator LArg)");



SwigHelpEntry("Functions","event sets"," EventSetUnion(alph1_arg, alph2_arg, alph_res) [depreciated; use AlphabetUnion instead]");
SwigHelpEntry("Functions","event sets"," EventSetIntersection(alph1_arg, alph2_arg, alph_res) [depreciated; use AlphabetIntersection instead]");
SwigHelpEntry("Functions","event sets"," EventSetDifference(alph1_arg, alph2_arg, alph_res) [depreciated; use AlphabetDifference instead]");

SwigHelpEntry("Functions","generator misc"," Version(+In+ Generator G, +In+ String Subscript, +Out+ Generator GRes)");
SwigHelpEntry("Functions","generator misc"," Version(+In+ Generator G, +In+ String Pattern, +In+ String Replacement, +Out+ Generator GRes)");

SwigHelpEntry("Functions","language misc"," ProjectNonDet(+InOut+ Generator LArg, +In+ EventSet Sigma0)");



