#!/bin/bash


# Script to record a test case
# 
# Notes:
# 1) run the script from the libfaudes root directory
# 2) specify the command to execute as a test relative to libfaudes root
# 3) results will be recorded for future test evaluation, incl a script to run the test
# 4) to unregister the test, remove its script

# configure
LIBFAUDES=.
LUAFAUDES=$LIBFAUDES/bin/luafaudes
COMMAND=$1
TESTDIR=$LIBFAUDES/tools/test
DIR=$(pwd)

# test/warn current directory
PWDBASE=$(basename $(pwd))
if test x$PWDBASE == x${PWDBASE#libfaudes} ; then { 
  echo error: you must make libfaudes the current directory to run this script;
  exit 1;
} fi

# derive file key by
# 1) replace / by _
# 2) replace . by _
# 3) replace multiple _ by one _
# 4) remove leading _
CMDKEY=$(echo $COMMAND | sed -e s/\\//_/g -e s/\\./_/g -e s/__*_/_/g -e s/^_//g)
echo using cmdkey $CMDKEY

# figure wehther the test is a lua script
if test $(basename $COMMAND .lua) == $(basename $COMMAND); then {
  echo assuming test to be a binary
  ISLUA=no
} else {
  echo assuming test to be a luafaudes script
  ISLUA=yes
} fi

# derive protocol by convention
if test $ISLUA == no; then {
  CMDBASE=$(basename $COMMAND)
  CMDDIR=$(dirname $COMMAND)
  PROTSRC=$CMDDIR/tmp_${CMDBASE}_cpp.prot
  PROTDST=$TESTDIR/${CMDKEY}_cpp.prot
  SCRIPT=$TESTDIR/${CMDKEY}_cpp.sh
} else {
  CMDBASE=$(basename $COMMAND)
  CMDSTRIP=$(basename $COMMAND .lua)
  CMDDIR=$(dirname $COMMAND)
  PROTSRC=$CMDDIR/tmp_${CMDSTRIP}_lua.prot
  PROTDST=$TESTDIR/$CMDKEY.prot
  SCRIPT=$TESTDIR/$CMDKEY.sh
} fi
echo expecting protocol src $PROTSRC
echo recording protocol dst $PROTDST
echo recording script at  $SCRIPT


# test/warn file existence: protocol
if test -r $PROTDST; then {
  echo warning: protocol exists
  echo press return to continue
  read
} fi

# test/warn file existence: script
if test -r $SCRIPT; then {
  echo warning: script exists
  echo press return to continue
  read
} fi



# execute
echo running test
cd $CMDDIR
if test $ISLUA == no; then {
  ./$CMDBASE
} else {
  $DIR/$LUAFAUDES $CMDBASE
} fi
cd $DIR
cp $PROTSRC $PROTDST
echo running test - done


# record test script
echo "# test script " >  $SCRIPT
echo "echo running test $COMMAND" >> $SCRIPT
echo "DIR=\$(pwd)" >>  $SCRIPT
echo "cd $CMDDIR" >>  $SCRIPT
if test $ISLUA == no; then {
  echo "./$CMDBASE &> /dev/null" >>  $SCRIPT
} else {
  echo "\$DIR/$LUAFAUDES $CMDBASE &> /dev/null" >>  $SCRIPT
} fi
echo "cd \$DIR"    >>  $SCRIPT
echo "diff -w --context=3 --show-function-line=mark $PROTSRC $PROTDST" >> $SCRIPT
chmod a+x $SCRIPT

# done
echo recorded script at  $SCRIPT
cat $SCRIPT
